import matplotlib.pyplot as plt
from mpl_toolkits.axisartist.axes_grid import ImageGrid
import lib.variables as V


class GeneratorVisualizer(object):

    def __init__(self, maps, trajectories, labels, val_result_dict):
        # val_results = dict(discriminated_bce_loss, batch_nb, pred_label, label, generated_alternative)
        self.alternatives = val_result_dict['generated_alternative']
        self.labels = labels
        self.trajectories = trajectories
        self.maps = maps
        self._map_width, self._map_height = self.maps[0].squeeze().shape
        self.column_dict_list = self._build_column_dict_list()
        self._cols = len(self.column_dict_list)
        self._rows = len(self.column_dict_list[0])

    def _build_column_dict_list(self):
        trajectories = []
        non_hom_alternatives = []
        hom_alternatives = []

        for idx in range(self.alternatives.shape[0]):
            image = (self.alternatives[idx]).cpu().numpy().squeeze()
            label = self.labels[idx].item()
            if label == V.HOMOTOPIC:
                hom_alternatives.append(dict(image=image, label='Homotopic'))
            else:
                non_hom_alternatives.append(dict(image=image, label='NonHomotopic'))
        for idx in range(max(len(hom_alternatives), len(non_hom_alternatives))):
            image = (self.maps[idx] + self.trajectories[idx]).cpu().numpy().squeeze()
            label = 'original'
            trajectories.append(dict(image=image, label=label))

        return trajectories, hom_alternatives, non_hom_alternatives

    def draw(self):
        padding = 0.25
        additional_size = self._cols * padding + 3 * padding
        width = (self._map_width * self._cols) / V.DPI + additional_size
        height = (self._map_height * self._rows) / V.DPI + additional_size
        fig = plt.figure(figsize=(width, height), dpi=V.DPI)
        grid = ImageGrid(fig, 111,  # similar to subplot(111)
                         nrows_ncols=(self._rows, self._cols),
                         axes_pad=padding,  # pad between axes in inch.
                         )

        for idx in range(len(grid.axes_all)):
            row, col = divmod(idx, len(self.column_dict_list))
            current_image = self.column_dict_list[col][row]['image']
            current_label = self.column_dict_list[col][row]['label']
            grid[idx].imshow(current_image)
            grid[idx].title.set_text(current_label)
        fig.cbar_mode = 'single'
        fig.tight_layout()
        return fig
